
#ifdef SINGLE_BLOCK

void mem_alloc(DWORD addr, DWORD size)
{
  mb.size = size;
  mb.addr = addr;
  mb.dataptr = new BYTE[size];
  memset(mb.dataptr, MEM_INIT, size);
  mb.flagptr = new DWORD[size];
  memset(mb.flagptr, FLAG_INIT, size * sizeof(DWORD));
}

void mem_load(DWORD addr, DWORD size, FILE* f, DWORD offs)
{
  fseek(f, offs, SEEK_SET);
  fread(DATAPTR(addr), sizeof(BYTE), size, f);
  for (DWORD n = 0; n < size; n++)
    FLAG(addr+n) |= FLAG_PRESENT;
}

#else // SINGLE_BLOCK

context::context()
{
  root = tail = curr = NULL;
}

void context::alloc(DWORD addr, DWORD size)
{
  memblock* t = new memblock;
  t->addr = addr;
  t->size = size;
  t->dataptr = new BYTE[size];
  memset(t->dataptr, MEM_INIT, size);
  t->flagptr = new DWORD[size];
  memset(t->flagptr, FLAG_INIT, size*sizeof(DWORD));
  t->next = NULL;
  if (root == NULL)
    curr = root = tail = t;
  else
    curr = tail = tail->next = t;
}

memblock* context::getblock(DWORD addr)
{
  if (addr - curr->addr < curr->size)
    return curr;
  for (memblock* t = root; t; t = t->next)
    if (addr - t->addr < t->size)
      return curr = t;
  return NULL;
}

BYTE* context::getdataptr(DWORD addr)
{
  if (getblock(addr))
    return &curr->dataptr[addr - curr->addr];
  else
    return NULL;
}

DWORD context::getsize(DWORD addr)
{
  if (getblock(addr))
    return curr->size - (addr - curr->addr);
  else
    return 0;
}

DWORD* context::getflagptr(DWORD addr)
{
  if (getblock(addr))
    return &curr->flagptr[addr - curr->addr];
  else
    return NULL;
}

BYTE context::getbyte(DWORD addr)
{
  if (getsize(addr) >= sizeof(BYTE))
    return *getdataptr(addr);
  else
    return MEMB_OUTOFRANGE;
}

void context::setbyte(DWORD addr, BYTE value)
{
  if (getsize(addr) >= sizeof(BYTE))
    *getdataptr(addr) = value;
}

WORD context::getword(DWORD addr)
{
  if (getsize(addr) >= sizeof(WORD))
    return *(WORD*)getdataptr(addr);
  else
    return MEMW_OUTOFRANGE;
}

void context::setword(DWORD addr, WORD value)
{
  if (getsize(addr) >= sizeof(WORD))
    *(WORD*)getdataptr(addr) = value;
}

DWORD context::getdword(DWORD addr)
{
  if (getsize(addr) >= sizeof(DWORD))
    return *(DWORD*)getdataptr(addr);
  else
    return MEMD_OUTOFRANGE;
}

void context::setdword(DWORD addr, DWORD value)
{
  if (getsize(addr) >= sizeof(DWORD))
    *(DWORD*)getdataptr(addr) = value;
}

DWORD context::getflag(DWORD addr)
{
  DWORD *flagptr = getflagptr(addr);
  return flagptr != NULL ? *flagptr : FLAG_OUTOFRANGE;
}

void context::setflag(DWORD addr, DWORD value)
{
  DWORD *flagptr = getflagptr(addr);
  if (flagptr != NULL)
    *flagptr = value;
}

void context::load(DWORD addr, DWORD size, FILE* f, DWORD offs)
{
  DWORD dsize = min(size, getsize(addr));
  BYTE* dptr = getdataptr(addr);
  DWORD* fptr = getflagptr(addr);
  fseek(f, offs, SEEK_SET);
  fread(dptr, sizeof(BYTE), dsize, f);
  for (DWORD n = 0; n < size; n++)
    fptr[n] |= FLAG_PRESENT;
}

#endif // SINGLE_BLOCK
